/*
 *      lark-gstreamer.h
 *
 *      Copyright 2016-2018 Alex <alex@linuxonly.ru>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LARK_GSTREAMER_H
#define LARK_GSTREAMER_H

#include <gtk/gtk.h>

#include "lark-app.h"

G_BEGIN_DECLS

#define LARK_TYPE_GSTREAMER (lark_gstreamer_get_type ())

G_DECLARE_FINAL_TYPE (LarkGstreamer, lark_gstreamer, LARK, GSTREAMER, GObject)

#define LARK_GSTREAMER_MIN_SAMPERATE        8000
#define LARK_GSTREAMER_MAX_SAMPERATE      125000
#define LARK_GSTREAMER_DEF_SAMPERATE       16000

#define LARK_GSTREAMER_MIN_CHANNELS            1
#define LARK_GSTREAMER_MAX_CHANNELS          127
#define LARK_GSTREAMER_DEF_CHANNELS            1

#define LARK_GSTREAMER_MIN_QUALITY             0
#define LARK_GSTREAMER_MAX_QUALITY            10
#define LARK_GSTREAMER_DEF_QUALITY             3

#define LARK_GSTREAMER_MIN_SIZE_LIMIT    1000000
#define LARK_GSTREAMER_MAX_SIZE_LIMIT  999000000
#define LARK_GSTREAMER_DEF_SIZE_LIMIT    1000000

#define LARK_GSTREAMER_MIN_TIME_LIMIT         60
#define LARK_GSTREAMER_MAX_TIME_LIMIT      36000
#define LARK_GSTREAMER_DEF_TIME_LIMIT         60

enum {
	LARK_GSTREAMER_SRC_FORMAT_U8 = 0,
	LARK_GSTREAMER_SRC_FORMAT_S8,
	LARK_GSTREAMER_SRC_FORMAT_U16LE,
	LARK_GSTREAMER_SRC_FORMAT_U16BE,
	LARK_GSTREAMER_SRC_FORMAT_S16LE,
	LARK_GSTREAMER_SRC_FORMAT_S16BE,
	LARK_GSTREAMER_SRC_FORMAT_U32LE,
	LARK_GSTREAMER_SRC_FORMAT_U32BE,
	LARK_GSTREAMER_SRC_FORMAT_S32LE,
	LARK_GSTREAMER_SRC_FORMAT_S32BE
};

enum {
	LARK_GSTREAMER_TARGET_AUDIO = 0,
	LARK_GSTREAMER_TARGET_WAV,
	LARK_GSTREAMER_TARGET_VORBIS,
	LARK_GSTREAMER_TARGET_OPUS,
	LARK_GSTREAMER_TARGET_FLAC,
	LARK_GSTREAMER_TARGET_MP3,
	LARK_GSTREAMER_TARGET_AAC
};

enum {
	LARK_GSTREAMER_EQ_FREQ_100 = 0,
	LARK_GSTREAMER_EQ_FREQ_300,
	LARK_GSTREAMER_EQ_FREQ_1K,
	LARK_GSTREAMER_EQ_FREQ_3K,
	LARK_GSTREAMER_EQ_FREQ_8K,
	LARK_GSTREAMER_EQ_FREQ_COUNT
};

enum {
	LARK_GSTREAMER_LIMIT_MODE_NONE = 0,
	LARK_GSTREAMER_LIMIT_MODE_SIZE,
	LARK_GSTREAMER_LIMIT_MODE_TIME
};

LarkGstreamer *lark_gstreamer_new(GError **error);

gboolean lark_gstreamer_add_data(LarkGstreamer *self, guint8 *data, gsize length);

gboolean lark_gstreamer_get_vorbis_available(LarkGstreamer *self);
gboolean lark_gstreamer_get_opus_available(LarkGstreamer *self);
gboolean lark_gstreamer_get_flac_available(LarkGstreamer *self);
gboolean lark_gstreamer_get_mp3_available(LarkGstreamer *self);
gboolean lark_gstreamer_get_aac_available(LarkGstreamer *self);

void lark_gstreamer_set_format(LarkGstreamer *self, gint format);
guint lark_gstreamer_get_format(LarkGstreamer *self);
void lark_gstreamer_set_sample_rate(LarkGstreamer *self, guint samplerate);
guint lark_gstreamer_get_sample_rate(LarkGstreamer *self);
void lark_gstreamer_set_channels(LarkGstreamer *self, guint channels);
guint lark_gstreamer_get_channels(LarkGstreamer *self);
void lark_gstreamer_set_target(LarkGstreamer *self, guint target);
guint lark_gstreamer_get_target(LarkGstreamer *self);
void lark_gstreamer_set_quality(LarkGstreamer *self, guint quality);
guint lark_gstreamer_get_quality(LarkGstreamer *self);
void lark_gstreamer_set_path(LarkGstreamer *self, gchar *path);
gchar *lark_gstreamer_get_path(LarkGstreamer *self);
void lark_gstreamer_set_file_name(LarkGstreamer *self, gchar *filename);
gchar *lark_gstreamer_get_file_name(LarkGstreamer *self);

void lark_gstreamer_set_limit_mode(LarkGstreamer *self, guint mode);
guint lark_gstreamer_get_limit_mode(LarkGstreamer *self);
void lark_gstreamer_set_size_limit(LarkGstreamer *self, guint sizelimit);
guint lark_gstreamer_get_size_limit(LarkGstreamer *self);
void lark_gstreamer_set_time_limit(LarkGstreamer *self, guint timelimit);
guint lark_gstreamer_get_time_limit(LarkGstreamer *self);

void lark_gstreamer_set_title_tag(LarkGstreamer *self, gchar *titletag);
gchar *lark_gstreamer_get_title_tag(LarkGstreamer *self);
void lark_gstreamer_set_author_tag(LarkGstreamer *self, gchar *authortag);
gchar *lark_gstreamer_get_author_tag(LarkGstreamer *self);
void lark_gstreamer_set_comment_tag(LarkGstreamer *self, gchar *commenttag);
gchar *lark_gstreamer_get_comment_tag(LarkGstreamer *self);

void lark_gstreamer_set_eq_gain(LarkGstreamer *self, guint freqid, gfloat gain);
gfloat lark_gstreamer_get_eq_gain(LarkGstreamer *self, guint freqid);

gsize lark_gstreamer_get_full_size(LarkGstreamer *self);
guint64 lark_gstreamer_get_full_duration(LarkGstreamer *self);

void lark_gstreamer_set_queue_bytes(LarkGstreamer *self, guint queuebytes);
guint lark_gstreamer_get_queue_bytes(LarkGstreamer *self);

gboolean lark_gstreamer_start(LarkGstreamer *self);

void lark_gstreamer_stop(LarkGstreamer *self, gboolean immediately);

G_END_DECLS

#endif /* LARK_GSTREAMER */
